/************************************************************************
 *                                                                      *
 * File:        my_proto.c		                                *
 *                                                                      *
 * Author(s):   M. Lewis, SRI International,                            *
 *              D. Beyer, Rooftop Communications.                       *
 *                                                                      *
 * Description: Simplified code example for a Radio "Protocol"          *
 *              or part of a device driver above the Radio API          *
 *                                                                      *
 ************************************************************************/

#include <stdio.h>
#include <my_rad.h>

typedef struct
{
    DevCntrlBlk controlBlk;

    uint32	otherStuffToKeepTrackOf;
    uint32 	forAParticularRadioDevice;
} ProtoDevType;

static ProtoDevType _ProtoDev;
static char         _RcvBuf[100];

void MySigCallback ( void * protoDev, uint32 num,     uint32 qual,
		     void * data,     uint32 dataLen, uint32 retCode );

void
main()
{
    uint32  	      retCode;
    uint32  	      qualifier;
    MyRadioPktInfo    packet;
    int32	      freq;	
    int		      i;	

    DevPktInfo	    * devPkt;
    DevCntrlBlk     * ctlBlk;		/* filled in by MyRadioInit */
    void 	    * radDev;
    byte              radVersion[100]; 

    /*
     * init and exchange context info with radio
     */

    ctlBlk  = &(_ProtoDev.controlBlk);

    retCode = MyRadioDevInit ( RADIO_API_VERSION, &_ProtoDev, DEV_ARG_PTR_NULL,
			       MySigCallback,  ctlBlk );
    if (retCode != RadioRetOk) exit(-1);

    radDev  = _ProtoDev.controlBlk.devHandle; 

    /*
     * print version
     */
    qualifier 	= RADIO_QUAL_GET;
    retCode = ctlBlk->var(radDev, RadioVarVersion, qualifier,
			  &radVersion, 100);
    if (retCode != RadioRetOk) exit(-2);

    printf ("\nExample skeletal radio device and protocol code.\n\n");
    printf ("Created by Rooftop Communications and SRI International.\n");
    printf ("to demonstrate GloMo Radio Device API.\n\n");
    printf ("Radio Version: \"%s\"\n\n", radVersion);    

    /*
     * attach to radio and assign name
     */

    retCode = ctlBlk->open(radDev, "yourRadio0", "");
    if (retCode != RadioRetOk) exit(-3);

    /*
     * reset Radio 
     */

    retCode   = ctlBlk->cmd(radDev,RadioCmdReset, 0, 
			    DEV_ARG_PTR_NULL, DEV_ARG_LEN_UNUSED);
    if (retCode != RadioRetOk) exit(-4);

    /*
     * give radio a receive buffer before enabling signals 
     */

    devPkt 	   = (DevPktInfo *)(&packet);
    devPkt->buf    = _RcvBuf;
    devPkt->bufLen = 100;

    retCode = ctlBlk->cmd(radDev, RadioCmdRcvPkt, 0,
			  &packet, sizeof(packet));
    if (retCode != RadioRetOk) exit(-5);

    /*
     * enable all radio signals to call my callback
     */

    retCode = ctlBlk->sigEnable(radDev, RadioSigAll, RadioSigEnable);
    if (retCode != RadioRetOk) exit(-6);

    /*
     * set Radio to desired frequency of 910 MHz 			
     * NOTE: we will standardized frequency to parameter mapping later
     */

    freq 	= 910000;
    qualifier 	= RADIO_QUAL_CH_ENCODE(1) | RADIO_QUAL_SET;
    retCode = ctlBlk->var(radDev, RadioVarFreq, qualifier,
			  &freq, DEV_ARG_LEN_INT32P);
    if (retCode != RadioRetOk) exit(-7);

    /*
     * transmit some packets
     */

    for (i = 0; i < 10; i++)
    { 
	devPkt 	       = (DevPktInfo *)(&packet);
	sprintf(devPkt->buf, "Loopback Pkt # %d", i);
	devPkt->bufLen = strlen (devPkt->buf);
    
	packet.radioPktInfo.macAdr = RADIO_MAC_BROADCAST_ADR;

	retCode = ctlBlk->cmd(radDev, RadioCmdXmtPkt,
			      0, &packet, sizeof(packet));
	if (retCode != RadioRetOk) exit(-8);
    }

    /*
     * close 
     */

    retCode = ctlBlk->close(radDev);
    if (retCode != RadioRetOk) exit(-9);
}

void
MySigCallback ( void      * protoDev,
	        uint32      num,
	        uint32      qual,
	        void      * data,
	        uint32      dataLen,
	        uint32      retCode )
{
    MyRadioPktInfo    packet;
    DevPktInfo      * devPkt;

    switch (num)
    {
    case RadioSigXmtPkt:
	/*
	 * my packet send is done
	 */

	break;

    case RadioSigRcvPkt:

	/*
	 * print out received packet
	 */

	printf ("Rcv pkt with contents: ");
	printf ("    %s\n", (char*) ((DevPktInfo *)data)->buf );
	
	/*
	 * hand back down the receive buffer
	 */

	devPkt	       = (DevPktInfo *)(&packet);
	devPkt->buf    = _RcvBuf;
	devPkt->bufLen = 100;

	retCode = _ProtoDev.controlBlk.cmd(_ProtoDev.controlBlk.devHandle,
					   RadioCmdRcvPkt, 0,
					   &packet, sizeof(packet));
	if (retCode != RadioRetOk) exit(-10);
	break;

    default:
        /*
         * ignore other signals
         */
    }
}

